#ifndef __LINUX_BOUNDS_H__
#define __LINUX_BOUNDS_H__
/*
 * DO NOT MODIFY.
 *
 * This file was generated by Kbuild
 */

#define NR_PAGEFLAGS 27 /* __NR_PAGEFLAGS */
#define MAX_NR_ZONES 5 /* __MAX_NR_ZONES */
#define NR_CPUS_BITS 9 /* order_base_2(CONFIG_NR_CPUS) */
#define SPINLOCK_SIZE 4 /* sizeof(spinlock_t) */
#define LRU_GEN_WIDTH 3 /* order_base_2(MAX_NR_GENS + 1) */
#define __LRU_REFS_WIDTH 2 /* MAX_NR_TIERS - 2 */

#endif
